import time
from selenium.webdriver.common.keys import Keys
from bs4 import BeautifulSoup
import pandas as pd
from tqdm import tqdm
from market_research.scraper._scraper_scheme import Scraper
from typing import Tuple
from tools.file.github import GitMgt


class Rurlsearcher(Scraper):
    def __init__(self, enable_headless=True, verbose=False):
        super().__init__(enable_headless=enable_headless)
        self.wait_time = 2
        self.verbose=  verbose

        
    def _get_model_info_from_mkrt(self, path_dict:dict=None) ->set:
        def get_recent_data_from_git(file_name):
            file_urls = []
            file_list = GitMgt.get_github_folder_files("xikest", "research_market_tv", "json")
            for file_url in file_list:
                if file_name in file_url:
                    file_urls.append(file_url)  
            file_urls.sort()          
            return file_urls[-1]
                
        if path_dict is None:
            path_dict = {
                    "sony_tv": f'{get_recent_data_from_git("s_scrape_model_data")}',
                    "lg_tv": f'{get_recent_data_from_git("l_scrape_model_data")}',
                    "samsung_tv": f'{get_recent_data_from_git("se_scrape_model_data")}',
                    "panasonic_tv": f'{get_recent_data_from_git("p_scrape_model_data")}',
                    "tcl_tv": f'{get_recent_data_from_git("t_scrape_model_data")}',
                    "sony_gaming": f'{get_recent_data_from_git("s_g_scrape_model_data")}',
                    "lg_gaming": f'{get_recent_data_from_git("l_g_scrape_model_data")}',
                    "samsung_gaming": f'{get_recent_data_from_git("se_g_scrape_model_data")}',
                    }
            info_df = pd.DataFrame()
        for maker in path_dict:
            df = pd.read_json(path_dict.get(maker), orient='records', lines=True)
            df = df[[ "year", "series"]]
            df.loc[:, "maker"] = maker
            info_df = pd.concat([info_df, df], axis=0)
        info_df =info_df.drop_duplicates()
        return info_df


    def get_urls_with_model_info(self) -> Tuple[list, pd.DataFrame]:
        info_df = self._get_model_info_from_mkrt()
        info_df = info_df.reset_index(drop=True)
        failed_series = []
        info_dict = {}
        for _, row in tqdm(info_df.iterrows()):
            
            maker = row['maker']
            series = row['series']
            year = row['year']
            try:
                keywords = f"{maker} {series}"
                search_query = f"{keywords +' '+ 'review'}"
                search_query = search_query.replace("gaming", "monitor").replace("_"," ")
                url_check = search_query.split()[1] ## "tv", "monitor"
                url = self._search_and_extract_url(search_query=search_query, url_check=url_check)
  
                check_keywords = f"{maker.split('_')[0]} {series}" ## "sony", "xr90"
                if self._check_url_with_keywords(url, check_keywords) is None:
                  raise ValueError
                info_dict[url] = {"maker":maker, 
                                  "series":series, 
                                  "year":year}               
            except:
                failed_series.append(series)
                continue
        if failed_series:
            print(f"failed_series: {failed_series}")
        return info_dict


    def _check_url_with_keywords(self, url: str, keywords: list):
        driver = self.web_driver.get_chrome()
        driver.get(url)

       

        try:
          page_source = driver.page_source
          soup = BeautifulSoup(page_source, 'html.parser')
          if soup.title and soup.title.string:
            title = soup.title.string.lower().replace("\u200b", "")
            if self.verbose:
                print(f"checking [{keywords}] in title[{title}]")##ss
            if all(keyword in title for keyword in keywords.split()):
              return url 
            else:
              raise ValueError
        except:
          return None
        finally:
          driver.quit()

    
    def get_urls_from_web(self, keywords: set = None) -> list:
        urls_set = set()
        for keyword in tqdm(keywords):
            url = self._search_and_extract_url(search_query=keyword)
            if url is not None:
                urls_set.add(url)
        urls_list = list(urls_set)        
        return urls_list
    


    def _search_and_extract_url(self, search_query: str, url_check:str=None, base_url="https://www.rtings.com"):
        driver = self.web_driver.get_chrome()
        if self.verbose:
                print(f"search_query: {search_query}")
                print(f"checking [{url_check}/review] in url") 
        try:
            driver.get(base_url)
            search_input = driver.find_element("class name", "searchbar-input")
            search_input.send_keys(search_query)
            search_input.send_keys(Keys.RETURN)
            time.sleep(self.wait_time)
            page_source = driver.page_source
            soup = BeautifulSoup(page_source, 'html.parser')
            search_results = soup.find_all('div', class_='searchbar_results-main')
            extracted_urls = [result.find('a')['href'] for result in search_results][:2]

            # 추출된 URL을 /로 분할하고, 검색 결과와 비교하여 일치하는 경우 반환
            if url_check:
              url_condition= f"{url_check}/reviews"
            else:
              url_condition= f"reviews"

            for url in extracted_urls:
                if url_condition in url:
                    split_url = url.split('/')
                    if len(split_url) == 5:
                        if self.verbose:
                            print(base_url + url)
                        return base_url + url
        except Exception as e:
            print(f"An error occurred: {e}")
            pass

        finally:
            # 브라우저 종료
            driver.quit()
        return None


from bs4 import BeautifulSoup
import requests
import time
import pandas as pd
from tqdm import tqdm
import re
import logging
from selenium.webdriver.common.by import By
from tools.file import FileManager
from market_research.scraper._scraper_scheme import Scraper, Modeler



class ModelScraper_l_g(Scraper, Modeler):
    def __init__(self, enable_headless=True,
                 export_prefix="lge_model_info_web", intput_folder_path="input", output_folder_path="results",
                 wait_time=2, verbose=False):

        Scraper.__init__(self, enable_headless, export_prefix, intput_folder_path, output_folder_path)
        self.wait_time = wait_time
        self.verbose = verbose
        pass

    def fetch_model_data(self) -> pd.DataFrame:
        
        def find_urls() -> dict:
            url_set = set()
            url_series_set = self._get_series_urls()
            for url in tqdm(url_series_set):
                url_models_set = self._extract_models_from_series(url=url)
                url_set.update(url_models_set)
            url_dict = {idx: url for idx, url in enumerate(url_set)}
            print(f"Total model: {len(url_dict)}")
            return url_dict
        
        def extract_sepcs(url_dict):
            dict_models = {}
            for key, url in tqdm(url_dict.items()):
                try:
                    dict_info = self._extract_model_details(url)
                    dict_models[key] = dict_info
                    dict_spec = self._extract_global_specs(url=url)
                    dict_spec['url'] = url
                    dict_models[key].update(dict_spec)
                except Exception as e:
                    if self.verbose:
                        print(f"fail to collect: {url}")
                        print(e)
            return dict_models
        
        def transform_format(dict_models, json_file_name: str) -> pd.DataFrame:
            df_models = pd.DataFrame.from_dict(dict_models).T
            df_models = df_models.dropna(subset=['price'])
            df_models.to_json(self.output_folder / json_file_name, orient='records', lines=True)
            return df_models
            
        print("start collecting data")
        url_dict = find_urls()
        dict_models = extract_sepcs(url_dict)
        df_models = transform_format(dict_models, json_file_name="l_g_scrape_model_data.json")
            
        FileManager.df_to_excel(df_models.reset_index(), file_name=self.output_xlsx_name)
        return df_models
    
    @Scraper.try_loop(2)
    def _get_series_urls(self) -> set:
        
        def click_load_more(driver):     
            for _ in range(5):
                try:
                    load_more_button = driver.find_element(By.XPATH, '/html/body/div[1]/div[3]/div[4]/div/div/div[2]/div/div/div[4]/div[2]/div[2]/button')
                    self.web_driver.move_element_to_center(load_more_button)
                    load_more_button.click()
                    time.sleep(1)
                except:
                    try:
                        load_more_button = driver.find_element(By.XPATH, '/html/body/div[1]/div[3]/div[3]/div/div/div[2]/div/div/div[4]/div[2]/div[2]/button')
                        self.web_driver.move_element_to_center(load_more_button)
                        load_more_button.click()
                        time.sleep(1)
                    except:
                        pass
                            
        def extract_urls_from_segments():
            def find_series_urls(url, prefix) -> set:
                print(f"Starting to scrape series URLs from: {url}")
                url_series = set()
                url = url
                prefix = prefix
                step = 200
                try:
                    driver = self.set_driver(url)
                    click_load_more()
                    scroll_distance_total = self.web_driver.get_scroll_distance_total()
                    scroll_distance = 0

                    while scroll_distance < scroll_distance_total:
                        for _ in range(2):
                            html = driver.page_source
                            soup = BeautifulSoup(html, 'html.parser')
                            elements = soup.find_all('a', class_="css-11xg6yi")
                            
                            for element in elements:
                                url_series.add(prefix + element['href'].strip())
                            driver.execute_script(f"window.scrollBy(0, {step});")
                            time.sleep(self.wait_time)
                            scroll_distance += step
                               
                    return url_series
                except Exception as e:
                    pass
                finally:
                    driver.quit()
            

            seg_urls = {
                "oled-gaming-monitors": "https://www.lg.com/us/ultragear-oled-gaming-monitors",
                "gaming-monitors": "https://www.lg.com/us/gaming-monitors",
                # 'gaming-monitors_0to50': 'https://www.lg.com/us/gaming-monitors?nf-ec_final_price_filter=0..50',
                # 'gaming-monitors_50to100': 'https://www.lg.com/us/gaming-monitors?nf-ec_final_price_filter=50..100',
                # 'gaming-monitors_100to150': 'https://www.lg.com/us/gaming-monitors?nf-ec_final_price_filter=100..150',
                # 'gaming-monitors_150to200': 'https://www.lg.com/us/gaming-monitors?nf-ec_final_price_filter=150..200',
                # 'gaming-monitors_200to250': 'https://www.lg.com/us/gaming-monitors?nf-ec_final_price_filter=200..250',
                # 'gaming-monitors_250to300': 'https://www.lg.com/us/gaming-monitors?nf-ec_final_price_filter=250..300',
                # 'gaming-monitors_300to350': 'https://www.lg.com/us/gaming-monitors?nf-ec_final_price_filter=300..350',
                # 'gaming-monitors_350to400': 'https://www.lg.com/us/gaming-monitors?nf-ec_final_price_filter=350..400',
                # 'gaming-monitors_400to450': 'https://www.lg.com/us/gaming-monitors?nf-ec_final_price_filter=400..450',
                # 'gaming-monitors_450to500': 'https://www.lg.com/us/gaming-monitors?nf-ec_final_price_filter=450..500',
                # 'gaming-monitors_500to600': 'https://www.lg.com/us/gaming-monitors?nf-ec_final_price_filter=500..600',
                # 'gaming-monitors_600to700': 'https://www.lg.com/us/gaming-monitors?nf-ec_final_price_filter=600..700',
                # 'gaming-monitors_700to800': 'https://www.lg.com/us/gaming-monitors?nf-ec_final_price_filter=700..800',
                # 'gaming-monitors_800to900': 'https://www.lg.com/us/gaming-monitors?nf-ec_final_price_filter=800..900',
                # 'gaming-monitors_900to1000': 'https://www.lg.com/us/gaming-monitors?nf-ec_final_price_filter=900..1000',
                # 'gaming-monitors_1000to1500': 'https://www.lg.com/us/gaming-monitors?nf-ec_final_price_filter=1000..1500',
                # 'gaming-monitors_1500to2000': 'https://www.lg.com/us/gaming-monitors?nf-ec_final_price_filter=1500..2000',
                # 'gaming-monitors_2000to3000': 'https://www.lg.com/us/gaming-monitors?nf-ec_final_price_filter=2000..3000'
            }
                
            
            url_series = set()
            
            for seg, seg_url in seg_urls.items():
                urls=find_series_urls(seg_url, prefix = "https://www.lg.com")
                url_series.update(urls)
            return url_series   
        
        url_series = extract_urls_from_segments( )
        print(f"The website scan has been completed.\ntotal series: {len(url_series)}")
        for i, url in enumerate(url_series, start=1):
            print(f"Series: [{i}] {url.split('/')[-1]}")
        return url_series
        
    @Scraper.try_loop(try_total=5)
    def _extract_models_from_series(self, url: str, prefix="https://www.lg.com/") -> set:
        return {url}

    @Scraper.try_loop(2)
    def _extract_model_details(self, url: str) -> dict:
        
        def extract_model(soup)->dict:
            
            label = soup.find('span', class_="MuiTypography-root MuiTypography-overline css-rrulv7").text.strip()
            model = label.split()[-1]
            return {"model": model}
        

       
        def extract_description(soup)->dict:
            descriptions = [
                ('h2', 'MuiTypography-root MuiTypography-subtitle2 css-8oa1vg'),
                ('h1', 'MuiTypography-root MuiTypography-subtitle2 css-8oa1vg'),
                ('h1', 'MuiTypography-root MuiTypography-h5 css-vnteo9'),
                ('h2','MuiTypography-root MuiTypography-h5 css-72m7wz')
            ]
            for tag, class_name in descriptions:
                try:
                    description = soup.find(tag, class_=class_name).text.strip()
                    if description: break  
                except:   
                    description =""
            return {"description": description}
        
        
        def extract_prices(soup, model_size)->dict:
            prices_dict = dict()
            try: 
                try:
                    price_now = soup.find('h6', class_='MuiTypography-root MuiTypography-subtitle1 css-1x0i2qf')
                    price_now = float(price_now.text.strip().replace('$', '').replace(',', ''))
                    price_original = soup.find('span', class_='MuiTypography-root MuiTypography-caption css-14jem7i')
                    price_original = float(price_original.text.strip().replace('$', '').replace(',', ''))
                except:
                    try:    
                        price_now = soup.find('h6', class_='MuiTypography-root MuiTypography-subtitle1 css-12myda0')
                        price_now = float(price_now.text.strip().replace('$', '').replace(',', ''))
                        try:
                            price_original = soup.find('span', class_='MuiTypography-root MuiTypography-caption css-14jem7i')
                            price_original = float(price_original.text.strip().replace('$', '').replace(',', ''))
                        except:
                            price_original = price_now

                    except:
                        products = {}
                        for item in soup.find_all('div', class_='MuiTypography-root MuiTypography-subtitle1 css-12myda0'):
                            size = item.find('h6').text.replace('"', '')
                            price = item.find('p').text  
                            products[size]= float(price.replace('$',"").replace(',', ''))
                        price_now = products.get(model_size)
                        prices_dict["price"] = price_now
                        price_original = price_now
                        prices_dict["price_gap"] = price_original - price_now
                            
                prices_dict["price"] = price_now
                prices_dict["price_original"] = price_original
                prices_dict["price_gap"] = price_original - price_now    
  
            except:
                prices_dict["price"] = float('nan')
                prices_dict["price_original"] = float('nan')
                prices_dict["price_gap"] = float('nan')
            return prices_dict
        
        def extract_info_from_model(model: str)->dict:
            model = model.lower()  # 대소문자 구분 제거
            dict_info = {}

            # 연도 매핑
            year_mapping =  {'k': "2018",
                            'l': "2019",
                            'n': "2022",
                            'p': "2021",
                            'q': "2022",
                            'r': "2023",
                            's': "2024",
                            'x':"2025"}
                        
            dict_info["size"] = model[:2]
            dict_info["grade"] = model[2]
            dict_info["series"] = model.split("-")[0]
            dict_info["year"] = model[3]
            dict_info["year"] = year_mapping.get(dict_info.get("year"), "0000")
            return dict_info

        
        dict_info = {}
        if self.verbose:
            print(f"Connecting to {url.split('/')[-1]}: {url}")
        logging.info(f"Connecting to {url.split('/')[-1]}: {url}")
        try:
            response = requests.get(url)
            page_content = response.text
            soup = BeautifulSoup(page_content, 'html.parser')
            
            dict_info.update(extract_model(soup))
            dict_info.update(extract_description(soup))
            dict_info.update(extract_info_from_model(dict_info.get("model")))
            dict_info.update(extract_prices(soup, dict_info.get('size')))
            if self.verbose: 
                print(dict_info)
            logging.info(dict_info)            
        except Exception as e:
            if self.verbose:
                print(f"error_extract_model_details: {url}")
            logging.error(f"error_extract_model_details: {url}")
            pass
        finally:
            return dict_info
    
    
    

    @Scraper.try_loop(try_total=10)
    def _extract_global_specs(self, url: str) -> dict:
      
        def find_spec_tab(driver):
            try: 
                for id in range(5):
                    element_all_specs = driver.find_element(By.ID, f"simple-tab-{id}")
                    if element_all_specs.text.lower() == "all specs":
                        break
                self.web_driver.move_element_to_center(element_all_specs)
                time.sleep(self.wait_time)
                element_all_specs.click()
                time.sleep(self.wait_time)
            except:
                try: 
                    element_all_specs = driver.find_element(By.CLASS_NAME, "MuiTypography-root.MuiTypography-h6.MuiTypography-alignLeft.MuiLink-root.MuiLink-underlineAlways.css-kgbp8r")
                    self.web_driver.move_element_to_center(element_all_specs)
                    time.sleep(self.wait_time)
                except:
                    element_all_specs = driver.find_element(By.CLASS_NAME, 'MuiTypography-root.MuiTypography-h5.css-14uiqdv')
                    self.web_driver.move_element_to_center(element_all_specs)
                    time.sleep(self.wait_time)
                
                
            return None 
        

        def extract_spec_detail(driver) -> dict:
            dict_spec = {}
            spec_elements = driver.find_elements(By.CSS_SELECTOR, '.MuiBox-root.css-1nnt9ji')
            for spec_element in spec_elements:
                try: 
                    label = spec_element.find_element(By.CLASS_NAME,
                                                        'MuiTypography-root.MuiTypography-body3.css-11mszpq').text.strip()
                    content = spec_element.find_element(By.CLASS_NAME,
                                                        'MuiTypography-root.MuiTypography-body2.css-1yx8hz4').text.strip()
                except:
 
                    label = spec_element.find_element(By.CLASS_NAME,
                                                    'MuiTypography-root.MuiTypography-body3.css-byc8c0').text.strip()
                    content = spec_element.find_element(By.CLASS_NAME,
                                                    'MuiTypography-root.MuiTypography-body2.css-1yx8hz4').text.strip()
                if label != '':
                    
                    original_label = label
                    while label in dict_spec and dict_spec.get(label)!=content:
                        asterisk_count = label.count('*')
                        label = f"{original_label}{'*' * (asterisk_count + 1)}"
                    dict_spec[label] = content
                    if self.verbose: 
                        if label == "" and content == "":
                            print(f"[{label}] {content}")

            dict_spec.pop("*", None)            
            return dict_spec
       
        dict_spec = dict()
        try:
            driver = self.set_driver(url)           
            find_spec_tab(driver)   
            dict_spec = extract_spec_detail(driver)
            if self.verbose:
                print(f"Received information from {url}")
            logging.info(f"Received information from {url}")
        except Exception as e:
            print(f"error extract_specs_detail from {url}")
            pass
        finally:
            if driver:  
                driver.quit()    
        return dict_spec
            
    def set_driver(self, url):
        driver = self.web_driver.get_chrome()
        driver.get(url=url)
        time.sleep(self.wait_time)
        return driver


from bs4 import BeautifulSoup
import time
import pandas as pd
from tqdm import tqdm
import re
import logging
from selenium.webdriver.common.by import By
from selenium.webdriver.common.action_chains import ActionChains
from market_research.scraper._scraper_scheme import Scraper, Modeler
from tools.file import FileManager
from selenium.webdriver.common.action_chains import ActionChains
from selenium.webdriver.common.keys import Keys

class ModelScraper_se_g(Scraper, Modeler):
    def __init__(self, enable_headless=True,
                 export_prefix="sse_model_info_web", intput_folder_path="input", output_folder_path="results",
                 wait_time=1, verbose=False):
        Scraper.__init__(self, enable_headless, export_prefix, intput_folder_path, output_folder_path)
        self.wait_time = wait_time
        self.verbose = verbose
        pass
    
    def fetch_model_data(self) -> pd.DataFrame:        
        def find_urls() -> dict:
            url_set = set()
            url_series_set = self._get_series_urls()

            
            for url in tqdm(url_series_set):
                url_models_set = self._extract_models_from_series(url=url)
                url_set.update(url_models_set)

            url_dict = {idx: url for idx, url in enumerate(url_set)}
            print(f"Total model: {len(url_dict)}")
            return url_dict
        
        def extract_sepcs(url_dict):
            dict_models = {}
            for key, url in tqdm(url_dict.items()):
                try:
                    dict_info = self._extract_model_details(url)
                    dict_models[key] = dict_info
                    dict_spec = self._extract_global_specs(url=url)
                    dict_spec['url'] = url
                    dict_models[key].update(dict_spec)
                except Exception as e:
                    if self.verbose:
                        print(f"fail to collect: {url}")
                        print(e)
                    pass
            return dict_models
        
        def transform_format(dict_models, json_file_name: str) -> pd.DataFrame:
            df_models = pd.DataFrame.from_dict(dict_models).T
            df_models = df_models.rename(columns={'Type':'display type'})
            df_models = df_models.dropna(subset=['price'])
            df_models.to_json(self.output_folder / json_file_name, orient='records', lines=True)
            return df_models
        print("start collecting data")
        url_dict = find_urls()
        dict_models = extract_sepcs(url_dict)
        
        df_models = transform_format(dict_models, json_file_name="se_g_scrape_model_data.json")
            
        FileManager.df_to_excel(df_models.reset_index(), file_name=self.output_xlsx_name)
        return df_models
    
    @Scraper.try_loop(2)
    def _get_series_urls(self) -> set:

        def extract_urls_from_segments():
            seg_urls = {
                "gaming_price_under200":"https://www.samsung.com/us/computing/monitors/gaming/?technology=Gaming&price=Under+%24200",
                "gaming_price_200to500":"https://www.samsung.com/us/computing/monitors/gaming/?technology=Gaming&price=%24200+-+%24500",
                "gaming_price_over500":"https://www.samsung.com/us/computing/monitors/gaming/?price=Over+%24500&technology=Gaming",
                "featured_gaming_monitors": "https://www.samsung.com/us/computing/monitors/gaming/?technology=Featured-gaming-monitors"
                }
            url_series = set()
            
            for seg, seg_url in seg_urls.items():
                urls=find_series_urls(seg_url, prefix = "https://www.samsung.com")
                url_series.update(urls)
            return url_series   
        

                
        def find_series_urls(url:str, prefix:str) -> set:
        
            print(f"Starting to scrape series URLs from: {url}")
            prefix = "https://www.samsung.com"
            url_series = set()
            url = url
            prefix = prefix
            step = 200

            try:
                driver = self.set_driver(url)
                scroll_distance_total = self.web_driver.get_scroll_distance_total()
                scroll_distance = 0
                
                while scroll_distance < scroll_distance_total:
                    for _ in range(2):
                        html = driver.page_source
                        soup = BeautifulSoup(html, 'html.parser')
                        elements = soup.find_all('a', class_="StarReview-review-1813701344 undefined")
                        for element in elements:
                            url_series.add(prefix + element['href'].strip())
                        driver.execute_script(f"window.scrollBy(0, {step});")
                        time.sleep(self.wait_time)
                        scroll_distance += step
                        tt= url.split('/')[-1]
                        driver.save_screenshot(f'ss/{tt}distance{scroll_distance}.png')    
                    
                        
                return url_series
            except Exception as e:
                if self.verbose:
                    print(f"error find_series_urls: {e}")
                pass
            finally:
                driver.quit()
               
        url_series = extract_urls_from_segments()

        print(f"The website scan has been completed.\ntotal series: {len(url_series)}")
        for i, url in enumerate(url_series, start=1):
            print(f"Series: [{i}] {url.split('/')[-2]}")
        return url_series
    
    @Scraper.try_loop(2)
    def _extract_models_from_series(self, url: str) -> set:
        
        def extract_url(url, XPATH = '//*[@id="details"]/div[2]/div[3]/div[2]/div[8]/div[1]/div[2]/div[2]') -> set:
            url_series_set= set()
            url_series_set.add(url)
            driver = self.set_driver(url)
            try:
                sereis_element = driver.find_elements(By.XPATH, XPATH)
                for element in sereis_element:
                    a_tags = element.find_elements(By.TAG_NAME, 'a')  
                    for a_tag in a_tags:
                        href = a_tag.get_attribute('href')  
                        if href:
                            href = href.strip()
                            url_series_set.add(href)
            except Exception as e:
                print(e)
            finally:
                driver.quit()              
                return url_series_set

        
        try: 
            url_models_set = set()
            series_url_set = extract_url(url,
                                         '//*[@id="details"]/div[2]/div[3]/div[2]/div[8]/div[1]/div[2]/div[2]')
            # print(f"series_url_set {len(series_url_set)}")
            for series_url in series_url_set:      
                url_models =  extract_url(series_url, 
                                              '//*[@id="details"]/div[2]/div[3]/div[2]/div[8]/div[1]/div[3]/div[2]')
                
                # if self.verbose:
                    # print(f"url_models of :{series_url}")
                    # print(f"url_models: {len(url_models)}")
                    # print(url_models)
                url_models_set.update(url_models)
        except Exception as e:
            if self.verbose:
                print(f"error_extract_models_from_series {url}")
        
        # if self.verbose:
            # print(url_models_set)
        return url_models_set
            
    @Scraper.try_loop(5)
    def _extract_model_details(self, url: str='') -> dict:
    
        def extract_model(driver):
            try:
                label_element = driver.find_element(By.XPATH,'//*[@id="details"]/div[2]/div[3]/div[2]/div[1]/div[1]/strong[2]')
                label = label_element.text
                model = label.split()[-1]
            except:
                try:
                    soup = BeautifulSoup(driver.page_source, 'html.parser')
                    label = soup.find('span', {'data-testid': 'atom_label'}).text.strip()
                    model = label.split('/')[-1].strip()
                except:
                    try:
                         title = driver.title
                         model = title.split('|')[0].split('-')[-1].strip()
                    except Exception as e:
                        if self.verbose:
                            print(e)
            return {"model": model}
            
        def extract_description(driver)->dict: 
            try:
                description = driver.find_element(By.XPATH,'//*[@id="details"]/div[2]/div[3]/div[2]/h1').text    

            except:
                soup = BeautifulSoup(driver.page_source, 'html.parser')

                try:
                    description = soup.find('div', class_='ProductTitle_product__q2vDb').text.strip()
                    description = description.split('/')[-1].strip()
                except:
                    try:
                         title = driver.title
                         description = title.split('-')[0].strip()
                    except Exception as e:
                        if self.verbose:
                            print(e)
                    
                    
            description = description.replace('"', "'") 
            return {"description": description}
             
        def extract_prices(driver) -> dict:
            prices_dict = {}
            soup = BeautifulSoup(driver.page_source, 'html.parser')
            try:
                price_info = soup.find('div', class_='PriceInfoText_priceInfo__QEjy8')
                price_now = float(price_info.find('b').text.strip().replace('$', '').replace(',', ''))
                price_original = price_now  
                strike_tag = price_info.find('strike')
                if strike_tag and strike_tag.text.strip():
                    price_original = float(strike_tag.text.strip().replace('$', '').replace(',', ''))
            except:
                try:
                    price_info = soup.find('span', class_='product-top-nav__font-desc noRecycle noTradein noNEO')
                    price_now = float(price_info.find('span', class_='epp-price price-color').text.strip().replace('$', '').replace(',', ''))
                    price_original = price_now  
                except Exception as e:
                    print(e)
                
            prices_dict["price"] = price_now
            prices_dict["price_original"] = price_original
            prices_dict["price_gap"] = price_original - price_now
            return prices_dict
            
        def extract_info_from_model(model: str)->dict:
            model = model.lower()  # 대소문자 구분 제거
            dict_info = {}

            # 연도 매핑
            year_mapping =  {'a': "2021",
                            'b': "2022",
                            'c': "2023",
                            'd': "2024"}
            dict_info["size"] = model[2:4]
            dict_info["year"] = model[4]
            dict_info["grade"] = model[:2]
            dict_info["series"] = model[1:8]
            dict_info["year"] = year_mapping.get(dict_info.get("year"), "0000")
            return dict_info
        
        dict_info = {}
        if self.verbose:
            print(f"Connecting to {url.split('/')[-2]}: {url}")
        logging.info(f"Connecting to {url.split('/')[-2]}: {url}")
        try: 
            driver = self.set_driver(url)
            
            # Extract model
            dict_info.update(extract_model(driver))
            dict_info.update(extract_description(driver))
            dict_info.update(extract_prices(driver))
            dict_info.update(extract_info_from_model(dict_info.get("model")))
            if self.verbose: 
                print(dict_info)
            logging.info(dict_info)
            
            
        except Exception as e:
            if self.verbose:
                print(f"error_extract_model_details: {url}")
                print(e)
            logging.error(f"error_extract_model_details: {url}")
            pass
        finally:
            if driver:  
                driver.quit()    
        return dict_info


    @Scraper.try_loop(3)
    def _extract_global_specs(self, url: str) -> dict:
        def find_spec_tab(driver) -> None:
            try: 
                # JavaScript로 'Specs' 링크 클릭
                driver.execute_script("arguments[0].click();", driver.find_element(By.ID, "#specsLink"))
                
                # 'See All Specs' 버튼 클릭
                driver.execute_script("arguments[0].click();", driver.find_element(By.CSS_SELECTOR, ".keyboard_navigateable.tl-btn-expand.Specs_expandBtn__0BNA_"))
                
                time.sleep(self.wait_time)
            except:
                try:
                    driver.execute_script("arguments[0].click();", driver.find_element(By.CSS_SELECTOR, "a[href='#specs']"))
                    # 'See All Specs' 버튼 클릭
                    driver.execute_script("arguments[0].click();", driver.find_element(By.CLASS_NAME, "tl-btn-expand"))
                
                except Exception as e :
                    if self.verbose:
                        print(f"error find_spec_tab {e}")
                    pass
                    time.sleep(self.wait_time)
                    
            return None 

        def extract_spec_detail(driver) -> dict:  
            dict_spec = {}
            try:
                table_elements = driver.find_elements(By.CLASS_NAME, "subSpecsItem.Specs_subSpecsItem__acKTN")
            except:
                try:
                    table_elements = driver.find_elements(By.CLASS_NAME, "Specs_specRow__e9Ife.Specs_specDetailList__StjuR")
                except:
                    try:
                        table_elements = driver.find_elements(By.CLASS_NAME, "spec-highlight__container")
                    except Exception as e:
                        print(f"error extract_spec_detail_click {e}")

            for element in table_elements:
                try:
                    item_name = element.find_element(By.CLASS_NAME, 'Specs_subSpecItemName__IUPV4').text
                    item_value = element.find_element(By.CLASS_NAME, 'Specs_subSpecsItemValue__oWnMq').text
                except:
                    try:
                        item_name = element.find_element(By.CLASS_NAME, 'Specs_subSpecItemName__IUPV4.Specs_type-p2__s07Sd').text
                        item_value = element.find_element(By.CLASS_NAME, 'Specs_type-p2__s07Sd.Specs_subSpecsItemValue__oWnMq').text 
                    except:
                        try:
                            item_name = element.find_element(By.CLASS_NAME, "spec-highlight__title").text
                            item_value = element.find_element(By.CLASS_NAME, "spec-highlight__value").text 
                        except Exception as e:
                            print(f"error extract_spec_detail_table {e}")  

                label = re.sub(r'[\n?]', '', item_name)
                content = re.sub(r'[\n?]', '', item_value)
                original_label = label
                while label in dict_spec and dict_spec.get(label)!=content:
                    asterisk_count = label.count('*')
                    label = f"{original_label}{'*' * (asterisk_count + 1)}"
                dict_spec[label] = content
                if self.verbose: 
                    if label == "" and content == "":
                        print(f"[{label}] {content}")
            return dict_spec
    
        dict_spec = {}
        try:
            driver = self.set_driver(url)
            find_spec_tab(driver)
            dict_spec = extract_spec_detail(driver)
            
            if self.verbose:
                print(f"Received information from {url}")
            logging.info(f"Received information from {url}")
        except Exception as e:
            print(f"error extract_specs_detail from {url}")
            pass
        finally:
            driver.quit()  
        return dict_spec



    def set_driver(self, url):
        driver = self.web_driver.get_chrome()
        driver.get(url=url)
        time.sleep(self.wait_time)
        return driver


import pandas as pd
import FinanceDataReader as fdr
from datetime import datetime
import plotly.graph_objects as go
from plotly.subplots import make_subplots


class MACRO:
    @staticmethod
    def plot_economic_indicator(key: str):
        today = datetime.now()
        start_date = today.replace(year=today.year - 5)  # 5년 전부터 시작
        end_date = today

        start_date_str = start_date.strftime('%Y-%m-%d')
        end_date_str = end_date.strftime('%Y-%m-%d')

        # FRED에서 데이터 가져오기
        data = fdr.DataReader(f'FRED:{key}', start=start_date_str, end=end_date_str)
        data = data.resample("QE").sum()  # 분기별 총합 계산

        # 전년 동기 데이터 생성
        data_previous = data.shift(4).iloc[-4:]  

        # 최근 4개 분기 데이터 필터링
        recent_data = data.iloc[-4:]

        fig = go.Figure()

        # 전년 동기 데이터 그래프
        fig.add_trace(go.Bar(
            x=data_previous.index,
            y=data_previous.squeeze(),
            name='Previous Year',
            opacity=0.5,
            showlegend=False,
            hovertemplate='%{y:.1f}<br>%{x|%Y-Q%q}' 
        ))

        # 현재 분기 데이터 그래프
        fig.add_trace(go.Bar(
            x=recent_data.index,
            y=recent_data.squeeze(),
            name='Current Year',
            showlegend=False,
            hovertemplate='%{x|%Y-Q%q}<br>Value: %{y:.2f}'  # 연도, 분기, 값 표시
        ))

        # 그래프 레이아웃 설정
        fig.update_layout(
            title=f'{key} - Comparison of Current Quarter and Previous Year',
            xaxis_title='',
            title_font=dict(size=10),
            margin=dict(t=25, b=0)
        )

        # X축 형식 설정
        fig.update_xaxes(
            tickvals=recent_data.index.tolist() + data_previous.index.tolist(),  # X축 값 설정
            ticktext=[f'Q{((x.month-1)//3)+1}' for x in recent_data.index] + 
                      [f'Q{((x.month-1)//3)+1}' for x in data_previous.index],  # 분기 형식으로 텍스트 설정
        )

        # Y축 로그 형식으로 설정
        fig.update_yaxes(type='log')

        return fig




import time
import pandas as pd
from market_research.scraper._scraper_scheme import Scraper
from tools.file.github import GitMgt
from selenium.webdriver.common.by import By
from bs4 import BeautifulSoup

class Erpsearcher(Scraper):
    def __init__(self, enable_headless=True, verbose=False):
        super().__init__(enable_headless=enable_headless)
        self.wait_time = 2
        self.verbose=  verbose

    def fetch_model_data(self):
        model_erp_data = [] 
        # info_models = self._get_model_info_from_mkrt()  
        info_df = self._get_model_info_from_mkrt() 

        info_models = zip(info_df['model'], info_df['maker'], info_df['price'],  info_df['size'], info_df['series'])

        for model, maker, price, size, series in info_models:
            
            brand_input = maker.split("_")[0]  
            model_erp_dict = self._search_data(model, brand_input)
            if model_erp_dict is None:
                continue
            model_erp_dict['maker'] = maker  
            model_erp_dict['query'] = model
            model_erp_dict['series'] = series
            model_erp_dict['size'] = size
            model_erp_dict['price'] = price

            model_erp_data.append(model_erp_dict)  
        
        model_erp_df = pd.DataFrame(model_erp_data)
        model_erp_df = model_erp_df.set_index("query").reset_index()
        model_erp_df.to_excel("model_erp_data.xlsx", index=False)
        model_erp_df.to_json(self.output_folder / "model_erp_data.json", orient='records', lines=True)
        return model_erp_df


        
    def _get_model_info_from_mkrt(self, path_dict:dict=None) ->dict:
        def get_recent_data_from_git(file_name):
            file_urls = []
            file_list = GitMgt.get_github_folder_files("xikest", "research_market_tv", "json")
            for file_url in file_list:
                if file_name in file_url:
                    file_urls.append(file_url)  
            file_urls.sort()          
            return file_urls[-1]
                
        if path_dict is None:
            path_dict = {
                    "sony_tv": f'{get_recent_data_from_git("s_scrape_model_data")}',
                    "lg_tv": f'{get_recent_data_from_git("l_scrape_model_data")}',
                    "samsung_tv": f'{get_recent_data_from_git("se_scrape_model_data")}',
                    "panasonic_tv": f'{get_recent_data_from_git("p_scrape_model_data")}',
                    "tcl_tv": f'{get_recent_data_from_git("t_scrape_model_data")}',
                    "sony_gaming": f'{get_recent_data_from_git("s_g_scrape_model_data")}',
                    "lg_gaming": f'{get_recent_data_from_git("l_g_scrape_model_data")}',
                    "samsung_gaming": f'{get_recent_data_from_git("se_g_scrape_model_data")}',
                    }
            info_df = pd.DataFrame()
        for maker in path_dict:
            df = pd.DataFrame()
            df_model = pd.read_json(path_dict.get(maker), orient='records', lines=True)
            if 'gaming' in maker.lower(): 
                df["model"] = df_model["series"]
            else:
                df["model"] = df_model["size"].astype(str) + df_model["series"].astype(str)

            df["price"] = df_model["price_original"]
            df["size"] = df_model["size"]
            df["series"] = df_model["series"]
            df.loc[:, "maker"] = maker
            info_df = pd.concat([info_df, df], axis=0)
        info_df =info_df.drop_duplicates()

        return info_df



    def _search_data(self, model_input:str, brand_input:str, base_url="https://eprel.ec.europa.eu/screen/product/electronicdisplays") ->dict:
        
        result = {}
        driver = self.web_driver.get_chrome()
        if self.verbose:
            print(f"search_query: {model_input}, {brand_input}")
        try:
            driver.get(base_url)
            for _ in range(5):
                try:
                    input_model_element = driver.find_element(By.ID, "model-identifier")
                    input_model_element.send_keys(model_input)
                    
                    input_supplier_element = driver.find_element(By.ID, "supplier-name")
                    input_supplier_element.send_keys(brand_input)
                    
                    search_button = driver.find_element(By.ID, "search")
                    driver.execute_script("arguments[0].click();", search_button)
                    time.sleep(self.wait_time)
                    search_result_items = driver.find_elements(By.TAG_NAME, "app-search-result-item")
                    break
                except:
                    continue

            for idx, item in enumerate(search_result_items):
                try:
                    supplier_name = item.find_element(By.XPATH, ".//span[contains(@class, 'ecl-u-type-2xl') and contains(@class, 'ecl-u-type-bold') and contains(@class, 'ecl-u-type-color-blue') and contains(@class, 'ecl-u-type-family-alt')]")
                    model_name = item.find_element(By.XPATH, ".//span[contains(@class, 'ecl-u-type-l') and contains(@class, 'ecl-u-type-color-black') and contains(@class, 'ecl-u-type-family-alt') and contains(@class, 'ecl-u-mt-xs')]")
                    
                    supplier_name = supplier_name.text.strip()
                    model_name = model_name.text.strip()

                       
                    if model_input.lower() in model_name.lower():
                        print(f"{supplier_name}: {model_name}")
                        result['brand']=supplier_name
                        result['model']=model_name
                        details_button = item.find_element(By.CSS_SELECTOR, ".ecl-u-width-100.ecl-button.ecl-button--primary")
                        driver.execute_script("arguments[0].click();", details_button)
                        if self.verbose:
                            driver.save_screenshot(f"click_on_details_of_{model_name}_with_{model_input}.png")
                        break
                except Exception as e:
                    print(e)
                   
            if 'brand' not in result or 'model' not in result:
                if self.verbose:
                    print(f"Model '{model_input}' not found in the search results.")
                return None
            
            time.sleep(self.wait_time)

            try:
                
                page_source = driver.page_source
                soup = BeautifulSoup(page_source, 'html.parser')
                
                title_element = soup.find('div', class_='ecl-u-media-bg-position-center')
                if title_element and title_element.get('title'):
                    title = title_element['title']
                    label = title[:-1]
                    grade = title[-1]
                    result[label] = grade.strip()
                    print(f"{label}: {grade}")
                else:
                    print('Energy class not found.')

                elements = soup.find_all('div', class_='ecl-u-align-items-l-end')
                for element in elements:
                    # 하위 요소 검색
                    key_element = element.find('div', class_='ecl-row')  # 키가 있는 요소
                    value_element = element.find('div', class_='ecl-u-flex-grow-0')  # 값이 있는 요소

                    # 키와 값 추출
                    key = key_element.get_text(strip=True).strip() if key_element else None
                    value = value_element.get_text(strip=True).replace("\n", " ").strip() if value_element else None

                    if key and value:  # 키와 값이 모두 존재하는 경우만 저장
                        result[key] = value
                        if self.verbose:
                            print(f"{key}: {value}")
            except Exception as e:
                print(f"Detail pages: {e}")
   
        except Exception as e:
            print(f"An error occurred: {e}")
            pass

        finally:
            driver.quit()
        return result



import time
from tqdm import tqdm
import pandas as pd
from selenium.webdriver.common.by import By
import logging
from tools.file import FileManager
from market_research.scraper._scraper_scheme import Scraper, Modeler


class ModelScraper_p(Scraper, Modeler):
    def __init__(self, enable_headless=True,
                 export_prefix="panasonic_model_info_web", intput_folder_path="input", output_folder_path="results",
                 wait_time=1 ,verbose=False):
        Scraper.__init__(self, enable_headless, export_prefix, intput_folder_path, output_folder_path)
        self.wait_time = wait_time
        self.verbose = verbose
        pass
    
    def fetch_model_data(self) -> pd.DataFrame:
        def find_urls() -> dict:
            url_set = set()
            url_series_set = self._get_series_urls()
            for url in url_series_set:
                url_models_set = self._extract_models_from_series(url=url)
                url_set.update(url_models_set)
            url_dict = {idx: url for idx, url in enumerate(url_set)}
            print(f"Total model: {len(url_dict)}")
            logging.info(f"Total model: {len(url_dict)}")
            return url_dict
        
        def extract_specs(url_dict):
            dict_models = {}
            for key, url in tqdm(url_dict.items()):
                try:
                    dict_info = self._extract_model_details(url)
                    dict_models[key] = dict_info
                    dict_spec = self._extract_global_specs(url=url)
                    dict_spec['url'] = url
                    dict_models[key].update(dict_spec)
                except Exception as e:
                    pass
            return dict_models
        
        def transform_format(dict_models, json_file_name: str) -> pd.DataFrame:
            df_models = pd.DataFrame.from_dict(dict_models).T
            df_models = df_models.dropna(subset=['price'])
            df_models.to_json(self.output_folder / json_file_name, orient='records', lines=True)
            return df_models
        
        
        print("start collecting data")
        logging.info("start collecting data")
        url_dict = find_urls()
        dict_models = extract_specs(url_dict)
        df_models = transform_format(dict_models, json_file_name="p_scrape_model_data.json")

        FileManager.df_to_excel(df_models.reset_index(), file_name=self.output_xlsx_name)
        return df_models
    
    @Scraper.try_loop(2)
    def _get_series_urls(self) -> set:
        def find_series_urls(url:str, prefix:str) -> set:
            print(f"Starting to scrape series URLs from: {url}")
            logging.info(f"Starting to scrape series URLs from: {url}")
            url_series = set()
            url = url
            prefix = prefix
            step = 200
            try:
                driver = self.set_driver(url)
                scroll_distance_total = self.web_driver.get_scroll_distance_total()
                scroll_distance = 0

                while scroll_distance < scroll_distance_total:
                    for _ in range(2):
                        elements = driver.find_elements(By.CLASS_NAME, "product-card__figure")
                        for element in elements:
                            try:
                                a_tag = element.find_element(By.TAG_NAME, 'a')  # `a` 태그를 찾음
                                link = a_tag.get_attribute('href')  # href 속성 가져오기
                                if link:  # 유효한 링크인지 확인
                                    url_series.add(link.strip())
                            except Exception as e:
                                print(f"Error extracting href: {e}")
                        driver.execute_script(f"window.scrollBy(0, {step});")
                        time.sleep(self.wait_time)
                        scroll_distance += step
                return url_series
            except Exception as e:
                if self.verbose:
                    print(f"{e}")
                logging.error(f"{e}")
                pass
            finally:
                if driver:  
                    driver.quit()                 
        url_series = find_series_urls(url = "https://shop.panasonic.com/collections/televisions", prefix = "https://shop.panasonic.com/")
        print(f"The website scan has been completed.\ntotal series: {len(url_series)}")
        logging.info(f"The website scan has been completed.\ntotal series: {len(url_series)}")
        for i, url in enumerate(url_series, start=1):
            print(f"Series: [{i}] {url.split('/')[-1]}")
            logging.info(f"Series: [{i}] {url.split('/')[-1]}")
        return url_series
    
    @Scraper.try_loop(2)
    def _extract_models_from_series(self, url: str) -> set:
        url_models_set = set()
        print(f"Trying to extract models from series: {url}")
        logging.info(f"Trying to extract models from series: {url}")
        
        driver = self.set_driver(url)        
        try: 
            elements = driver.find_elements(By.CLASS_NAME, 'block-swatch')
            
            for element in elements:
                element.click()  # 
                time.sleep(2)  #
                current_url = driver.current_url
                if current_url:  
                    url_models_set.add(current_url.strip())
            print("Extracted models from series")
            logging.info(f"Trying to extract models from series: {url}")
        except Exception as e:
            if self.verbose:
                print(f"Error extracting models from series: {e}")
            logging.error(f"Error extracting models from series: {e}")
        finally:
            if driver:  
                driver.quit()  
                
        return url_models_set
        
    
            
    @Scraper.try_loop(2)
    def _extract_model_details(self, url: str) -> dict:
        
        def extract_model(driver):
            model = driver.find_element(By.CSS_SELECTOR, ".product-info__sku.text-16.text-subdued")
            model = model.text.split(":")[-1].strip()
            return {"model": model}
        
        def extract_description(driver)->dict:
            try:
                description= driver.find_element(By.CSS_SELECTOR, "h1.product-info__title").text
            except:
                description = ""
            return {"description": description}
        
        def extract_prices(driver)->dict:
            prices_dict = dict()

            try:
                price_now = driver.find_element(By.CSS_SELECTOR, "price-list .text-lg").text
                price_now = price_now.split("$")[-1]
                price_now = float(price_now.replace('$', '').replace(',', ''))
                prices_dict['price'] = price_now
                prices_dict['price_original'] = price_now
                prices_dict['price_gap'] = 0.0
            except Exception as e:
                print(e)
                prices_dict['price'] = float('nan')
                prices_dict['price_original'] = float('nan')
                prices_dict['price_gap'] = float('nan')
            return prices_dict
        
        def extract_info_from_model(model: str)->dict:
            dict_info = {}
            model = model.lower()
            dict_info["model"] = model
            dict_info["year"] = model.split("-")[-1]
            dict_info["series"] = model.split("-")[1][2:]
            dict_info["size"] = model.split("-")[1][:2]
            dict_info["display type"] = dict_info["series"][:1]
                
            year_mapping = {
                't': "2024",
            }
            display_mapping = {
                'z':'oled',
                'w':'mini-LED'
            }
            dict_info["year"] = year_mapping.get(dict_info.get("year"), "2025")
            dict_info["display type"] = display_mapping.get(dict_info.get("display type"), "")
            return dict_info
        
        dict_info = {}
        if self.verbose:
            print(f"Connecting to {url.split('/')[-1]}: {url}")
        logging.info(f"Connecting to {url.split('/')[-1]}: {url}")
        try:
            driver = self.set_driver(url)
            
            # Extract model
            dict_info.update(extract_model(driver))
            dict_info.update(extract_description(driver))
            dict_info.update(extract_prices(driver))
            dict_info.update(extract_info_from_model(dict_info.get("model")))
            if self.verbose: 
                print(dict_info)
            logging.info(dict_info)
        except Exception as e:
            print(e)
            if self.verbose:
                print(f"error_extract_model_details: {url}")
            logging.error(f"error_extract_model_details: {url}")
            pass
        finally:
            if driver:  
                driver.quit()  
        return dict_info
    
    @Scraper.try_loop(5)
    def _extract_global_specs(self, url: str) -> dict:
        
        
        def find_spec_tab(driver) -> None:
            def act_click_see_more(driver):
                try: 
                    element = driver.find_element(By.CLASS_NAME,'feature-chart__toggle')
                    element_see_more = element.find_element(By.CLASS_NAME, 'text-with-icon.group')
                    # self.web_driver.move_element_to_center(element_see_more)
                    element_see_more.click()
                    logging.debug("Clicked the 'see more' button from the first locator.")
                    driver.save_screenshot("41act_click_see_more.png")                
                except Exception as e:
                    if self.verbose:
                        print(f"Cannot find the 'see more' button from the second locator: {e}")
                        logging.error(f"Cannot find the 'see more' button from the second locator: {e}")
                time.sleep(self.wait_time)
                return None
            
            act_click_see_more(driver)       
            return None
            
        def extract_specs_detail(driver) -> dict:
            dict_spec = {}
            table = driver.find_element(By.CLASS_NAME, 'section-stack.section-stack--horizontal ')
            rows = table.find_elements(By.CLASS_NAME, 'feature-chart__table-row')
            for row in rows:
                text = row.text.split('\n')
                label = text[0]
                value = text[-1] 
                dict_spec[label] = value        
            return dict_spec
        
        dict_specs = {}
            
        try:
            driver = self.set_driver(url)
            find_spec_tab(driver)   
            dict_specs.update(extract_specs_detail(driver))
            if self.verbose:
                print(f"Received information from {url}")
            logging.info(f"Received information from {url}")
        except Exception as e:
            print(f"error extract_specs_detail from {url}")
            print(e)
            pass
        finally:
            if driver:  
                driver.quit()  
        return dict_specs
        
        
    def set_driver(self, url):
        driver = self.web_driver.get_chrome()
        driver.get(url=url)
        time.sleep(self.wait_time)
        return driver
